@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2003 Thorsten Kukuk
+/* Copyright (C) 2001, 2003, 2014 Thorsten Kukuk
    This file is part of the yp-tools.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -26,9 +26,9 @@
 #include <string.h>
 #include <unistd.h>
 #include <rpc/rpc.h>
-#include <rpcsvc/yp.h>
-#include <rpcsvc/ypclnt.h>
 #include <arpa/inet.h>
+#include <rpcsvc/ypclnt.h>
+#include <rpcsvc/yp_prot.h>
 #include "yp_all_host.h"
 
 static struct timeval RPCTIMEOUT = {10, 0};
@@ -58,23 +58,23 @@ __xdr_ypresp_all (XDR *xdrs, u_long *objp)
           return TRUE;
         }
 
-      switch (resp.ypresp_all_u.val.stat)
+      switch (resp.ypresp_all_u.val.status)
         {
         case YP_TRUE:
           {
-            char key[resp.ypresp_all_u.val.key.keydat_len + 1];
-            char val[resp.ypresp_all_u.val.val.valdat_len + 1];
-            int keylen = resp.ypresp_all_u.val.key.keydat_len;
-            int vallen = resp.ypresp_all_u.val.val.valdat_len;
+            char key[resp.ypresp_all_u.val.keydat.keydat_len + 1];
+            char val[resp.ypresp_all_u.val.valdat.valdat_len + 1];
+            int keylen = resp.ypresp_all_u.val.keydat.keydat_len;
+            int vallen = resp.ypresp_all_u.val.valdat.valdat_len;
 
             /* We are not allowed to modify the key and val data.
                But we are allowed to add data behind the buffer,
                if we don't modify the length. So add an extra NUL
                character to avoid trouble with broken code. */
             *objp = YP_TRUE;
-            memcpy (key, resp.ypresp_all_u.val.key.keydat_val, keylen);
+            memcpy (key, resp.ypresp_all_u.val.keydat.keydat_val, keylen);
             key[keylen] = '\0';
-            memcpy (val, resp.ypresp_all_u.val.val.valdat_val, vallen);
+            memcpy (val, resp.ypresp_all_u.val.valdat.valdat_val, vallen);
             val[vallen] = '\0';
             xdr_free ((xdrproc_t) xdr_ypresp_all, (char *) &resp);
             if ((*ypall_foreach) (*objp, key, keylen,
@@ -83,7 +83,7 @@ __xdr_ypresp_all (XDR *xdrs, u_long *objp)
           }
           break;
         default:
-          *objp = resp.ypresp_all_u.val.stat;
+          *objp = resp.ypresp_all_u.val.status;
           xdr_free ((xdrproc_t) xdr_ypresp_all, (char *) &resp);
           /* Sun says we don't need to make this call, but must return
              immediatly. Since Solaris makes this call, we will call
@@ -95,16 +95,14 @@ __xdr_ypresp_all (XDR *xdrs, u_long *objp)
 }
 
 int
-yp_all_host (const char *hostname, const char *indomain, const char *inmap,
-	     const struct ypall_callback *incallback)
+yp_all_host (const char *indomain, const char *inmap,
+	     const struct ypall_callback *incallback, const char *hostname)
 {
   struct ypreq_nokey req;
   int res;
   enum clnt_stat result;
-  struct sockaddr_in clnt_sin;
   CLIENT *clnt;
   long status = 0;
-  int clnt_sock;
 
   if (hostname == NULL || hostname[0] == '\0' ||
       indomain == NULL || indomain[0] == '\0' ||
@@ -113,20 +111,7 @@ yp_all_host (const char *hostname, const char *indomain, const char *inmap,
 
   res = YPERR_YPERR;
 
-  clnt_sock = RPC_ANYSOCK;
-
-  if (inet_aton (hostname, &clnt_sin.sin_addr) == 0)
-    {
-      struct hostent *host  = gethostbyname (hostname);
-      if (!host)
-	return YPERR_BADARGS;
-      memcpy (&clnt_sin.sin_addr, host->h_addr_list[0],
-	      sizeof (clnt_sin.sin_addr));
-    }
-  clnt_sin.sin_family = AF_INET;
-
-  clnt_sin.sin_port = 0;
-  clnt = clnttcp_create (&clnt_sin, YPPROG, YPVERS, &clnt_sock, 0, 0);
+  clnt = clnt_create_timed (hostname, YPPROG, YPVERS, "tcp", NULL);
   if (clnt == NULL)
     return YPERR_PMAP;
 
