@@ -20,25 +20,15 @@
 #include "config.h"
 #endif
 
-#define _GNU_SOURCE
-
 #define keydat_val dptr
 #define keydat_len dsize
 #define valdat_val dptr
 #define valdat_len dsize
 
 #include <pwd.h>
-#ifdef HAVE_XCRYPT_H
-#include <xcrypt.h>
-#elif defined (HAVE_CRYPT_H)
 #include <crypt.h>
-#endif
 #include <ctype.h>
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#else
-#include "lib/getopt.h"
-#endif
 #include <errno.h>
 #include <time.h>
 #include <netdb.h>
@@ -54,16 +44,7 @@
 #include <fcntl.h>
 #include <rpcsvc/yp_prot.h>
 #include <rpcsvc/ypclnt.h>
-#ifdef HAVE_RPC_CLNT_SOC_H
-#include <rpc/clnt_soc.h>
-#endif
-
-#include "yppasswd.h"
-
-#if defined (__NetBSD__) || (defined(__GLIBC__) && (__GLIBC__ == 2 && __GLIBC_MINOR__ == 0))
-/* <rpc/rpc.h> is missing the prototype */
-int getrpcport(char *host, int prognum, int versnum, int proto);
-#endif
+#include <rpcsvc/yppasswd.h>
 
 #ifndef _
 #define _(String) gettext (String)
@@ -629,33 +610,6 @@ main (int argc, char **argv)
 		     strerror (errno));
 	}
     }
-
-  if (l_flag)
-    {
-      struct stat st;
-      if (stat (CHSH_PROG, &st) == 0)
-	{
-	  printf (_("ypchsh is deprecated, use %s instead\n\n"),
-		  CHSH_PROG);
-	  if (execl (CHSH_PROG, "chsh", user, NULL) == -1)
-	    fprintf (stderr, _("Calling %s failed: %s\n"), CHSH_PROG,
-		     strerror (errno));
-	}
-    }
-
-  if (f_flag)
-    {
-      struct stat st;
-      if (stat (CHFN_PROG, &st) == 0)
-	{
-	  printf (_("ypchfn is deprecated, use %s instead\n\n"),
-		  CHFN_PROG);
-
-	  if (execl (CHFN_PROG, "chfn", user, NULL) == -1)
-	    fprintf (stderr, _("Calling %s failed: %s\n"), CHFN_PROG,
-		     strerror (errno));
-	}
-    }
 #endif
 
   if ((error = yp_get_default_domain (&domainname)) != 0)
