@@ -1,4 +1,4 @@
-/* Copyright (C) 1998, 1999, 2001 Thorsten Kukuk
+/* Copyright (C) 1998, 1999, 2001, 2014 Thorsten Kukuk
    This file is part of the yp-tools.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -20,30 +20,31 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#else
-#include "lib/getopt.h"
-#endif
 #include <locale.h>
 #include <libintl.h>
 #include <netdb.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <string.h>
 #include <rpc/rpc.h>
-#ifdef HAVE_RPC_CLNT_SOC_H
-#include <rpc/clnt_soc.h>
-#endif
-#include "lib/yp-tools.h"
+#include <rpcsvc/yp_prot.h>
 #include "lib/nicknames.h"
 
+/* from ypbind-mt/ypbind.h */
+#define YPBINDPROC_OLDDOMAIN 1
+extern int yp_maplist (const char *, struct ypmaplist **);
+
 #ifndef _
 #define _(String) gettext (String)
 #endif
 
+/* --verbose option */
+static int verbose = 0;
+
 /* Name and version of program.  */
 /* Print the version information.  */
 static void
@@ -54,7 +55,7 @@ print_version (void)
 Copyright (C) %s Thorsten Kukuk.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "1998");
+"), "2014");
   /* fprintf (stdout, _("Written by %s.\n"), "Thorsten Kukuk"); */
 }
 
@@ -76,9 +77,10 @@ print_help (void)
   fputs (_("  -m mname       Find the master NIS server for the map 'mname'\n"),
 	 stdout);
   fputs (_("  -t             Inhibits map nickname translation\n"), stdout);
-  fputs (_("  -V n           Version of ypbind, V2 is default\n"), stdout);
+  fputs (_("  -V n           Version of ypbind, V3 is default\n"), stdout);
   fputs (_("  -x             Display the map nickname translation table\n"),
 	 stdout);
+  fputs (_(" --verbose       Verbose output of result\n"), stdout);
   fputs (_("  -?, --help     Give this help list\n"), stdout);
   fputs (_("      --usage    Give a short usage message\n"), stdout);
   fputs (_("      --version  Print program version\n"), stdout);
@@ -98,61 +100,175 @@ print_error (void)
 /* bind to a special host and print the name ypbind running on this host
    is bound to */
 static int
-print_bindhost (char *domain, struct sockaddr_in *socka_in, int vers)
+print_bindhost (char *hostname, char *domain, int vers)
 {
-  struct hostent *host = NULL;
-  struct ypbind_resp yp_r;
+  int ret;
   struct timeval tv;
   CLIENT *client;
-  int sock, ret;
-  struct in_addr saddr;
 
-  sock = RPC_ANYSOCK;
-  tv.tv_sec = 5;
-  tv.tv_usec = 0;
-  client = clntudp_create (socka_in, YPBINDPROG, vers, tv, &sock);
+  client = clnt_create(hostname, YPBINDPROG, (vers==-1)?3:vers, "udp");
   if (client == NULL)
     {
       fprintf (stderr, "ypwhich: %s\n", yperr_string (YPERR_YPBIND));
+      if (verbose)
+	{
+	  fprintf(stderr, "Error calling clnt_create()\n");
+	  fprintf(stderr, "PROG: %i\tVERS: %i\tNET: %s\n",
+		  YPBINDPROG, vers, "upd");
+	  fprintf(stderr, "clnt_stat: %d\n", rpc_createerr.cf_stat);
+	  fprintf(stderr, "re_errno: %d\n", rpc_createerr.cf_error.re_errno);
+	}
       return 1;
     }
 
   tv.tv_sec = 15;
   tv.tv_usec = 0;
-  if (vers == 1)
-    ret = clnt_call (client, YPBINDPROC_OLDDOMAIN,
-		     (xdrproc_t) ytxdr_domainname,
-		     (caddr_t) &domain, (xdrproc_t) ytxdr_ypbind_resp,
-		     (caddr_t) &yp_r, tv);
-  else
-    ret = clnt_call (client, YPBINDPROC_DOMAIN, (xdrproc_t) ytxdr_domainname,
-		     (caddr_t) &domain, (xdrproc_t) ytxdr_ypbind_resp,
-		     (caddr_t) &yp_r, tv);
 
-  if (ret != RPC_SUCCESS)
+  if (vers < 3 && vers > 0)
     {
-      fprintf (stderr, "ypwhich: %s\n", yperr_string (YPERR_YPBIND));
+      struct ypbind2_resp yp2_r;
+      struct sockaddr_in sa;
+      char host[NI_MAXHOST];
+
+      memset (&yp2_r, 0, sizeof (struct ypbind2_resp));
+
+      if (vers == 1)
+	ret = clnt_call (client, YPBINDPROC_OLDDOMAIN,
+			 (xdrproc_t) xdr_domainname,
+			 (caddr_t) &domain, (xdrproc_t) xdr_ypbind2_resp,
+			 (caddr_t) &yp2_r, tv);
+      else
+	ret = clnt_call (client, YPBINDPROC_DOMAIN, (xdrproc_t) xdr_domainname,
+			 (caddr_t) &domain, (xdrproc_t) xdr_ypbind2_resp,
+			 (caddr_t) &yp2_r, tv);
+
+      if (ret != RPC_SUCCESS)
+	{
+	  char *err = NULL;
+
+	  if (asprintf (&err, _("ypwhich: can't call ypbind on '%s'\n\t"),
+			hostname) > 0)
+	    {
+	      clnt_perror (client, err);
+	      free (err);
+	    }
+	  clnt_destroy (client);
+	  return 1;
+	}
+      else
+	{
+	  if (yp2_r.ypbind_status != YPBIND_SUCC_VAL ||
+	      /* that's an ugly hack, but ypbind of Solaris reports for V1 and V2
+		 YPBIND_SUCC_VAL if it is bound to a server with IPv6 address. */
+	      yp2_r.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port == 0)
+	    {
+	      fprintf (stderr,
+		       _("Error for domain '%s' from ypbind on '%s':\n\t%s\n"),
+		       domain, hostname,
+		       ypbinderr_string (yp2_r.ypbind_respbody.ypbind_error));
+	      clnt_destroy (client);
+	      return 1;
+	    }
+	}
       clnt_destroy (client);
-      return 1;
-    }
-  else
-    {
-      if (yp_r.ypbind_status != YPBIND_SUCC_VAL)
-        {
-          fprintf (stderr, _("can't yp_bind: Reason: %s\n"),
-                   ypbinderr_string (yp_r.ypbind_respbody.ypbind_error));
-          clnt_destroy (client);
-          return 1;
-        }
-    }
-  clnt_destroy (client);
 
-  saddr = yp_r.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr;
-  host = gethostbyaddr ((char *) &saddr, sizeof (saddr), AF_INET);
-  if (host)
-    printf ("%s\n", host->h_name);
-  else
-    printf ("%s\n", inet_ntoa (saddr));
+      sa.sin_family = AF_INET;
+      sa.sin_addr =  yp2_r.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr;
+
+      if (getnameinfo((struct sockaddr *)&sa, sizeof sa,
+		      host, sizeof host, NULL, 0, 0) == 0)
+	printf ("%s\n", host);
+      else
+	{
+	  char straddr[INET_ADDRSTRLEN];
+	  inet_ntop(sa.sin_family, &sa.sin_addr, straddr, sizeof(straddr));
+	  printf ("%s\n", straddr);
+	}
+    }
+  else /* YPBINDVERS >= 3 || -1 */
+    {
+      struct ypbind3_resp yp3_r;
+
+      memset (&yp3_r, 0, sizeof (struct ypbind3_resp));
+
+      ret = clnt_call (client, YPBINDPROC_DOMAIN, (xdrproc_t) xdr_domainname,
+		       (caddr_t) &domain, (xdrproc_t) xdr_ypbind3_resp,
+		       (caddr_t) &yp3_r, tv);
+
+      if (ret != RPC_SUCCESS)
+	{
+	  char *err = NULL;
+
+	  /* if we have a RPC version mismatch, try version 2 */
+	  if (ret == RPC_PROGVERSMISMATCH && vers == -1)
+	    return print_bindhost (hostname, domain, 2);
+
+	  if (asprintf (&err, _("ypwhich: can't call ypbind on '%s'\n\t"),
+			hostname) > 0)
+	    {
+	      clnt_perror (client, err);
+	      free (err);
+	    }
+	  clnt_destroy (client);
+	  return 1;
+	}
+      else
+	{
+	  if (yp3_r.ypbind_status != YPBIND_SUCC_VAL)
+	    {
+	      fprintf (stderr,
+		       _("Error for domain '%s' from ypbind on '%s':\n\t%s\n"),
+		       domain, hostname,
+		       ypbinderr_string (yp3_r.ypbind3_error));
+	      clnt_destroy (client);
+	      return 1;
+	    }
+	}
+      clnt_destroy (client);
+
+      if (verbose)
+	{
+	  char namebuf6[INET6_ADDRSTRLEN];
+
+	  if (yp3_r.ypbind3_nconf && yp3_r.ypbind3_svcaddr)
+	    printf ("ypbind_netbuf:\n\taddr: %s\n\tport: %i\n",
+		    taddr2ipstr (yp3_r.ypbind3_nconf,
+				 yp3_r.ypbind3_svcaddr,
+				 namebuf6, sizeof namebuf6),
+		    taddr2port (yp3_r.ypbind3_nconf, yp3_r.ypbind3_svcaddr));
+
+	  if (yp3_r.ypbind3_servername && strlen (yp3_r.ypbind3_servername) > 0)
+	    printf ("ypbind_servername: %s\n", yp3_r.ypbind3_servername);
+	  else
+	    printf ("ypbind_servername: NULL\n");
+	  printf ("ypbind_hi_vers: %u\n", (u_int32_t)yp3_r.ypbind3_hi_vers);
+	  printf ("ypbind_lo_vers: %u\n", (u_int32_t)yp3_r.ypbind3_lo_vers);
+	}
+      else if (yp3_r.ypbind3_servername && strlen (yp3_r.ypbind3_servername) > 0)
+	printf ("%s\n", yp3_r.ypbind3_servername);
+      else if (yp3_r.ypbind3_nconf && yp3_r.ypbind3_svcaddr)
+	{
+	  char hostbuf[NI_MAXHOST];
+	  const char *host;
+
+	  host = taddr2host (yp3_r.ypbind3_nconf, yp3_r.ypbind3_svcaddr,
+			     hostbuf, sizeof hostbuf);
+
+	  if (host)
+	    printf ("%s\n", host);
+	  else
+	    {
+	      fprintf (stderr, _("ERROR: taddr2host failed!\n"));
+	      return 1;
+	    }
+	}
+      else
+	{
+	  fprintf (stderr, _("Error: no server information gotten from ypbind on '%s'\n"),
+		   hostname);
+	  return 1;
+	}
+    }
   return 0;
 }
 
@@ -162,7 +278,7 @@ main (int argc, char **argv)
 {
   int dflag = 0, mflag = 0, tflag = 0, Vflag = 0, xflag = 0, hflag = 0;
   char *hostname = NULL, *domainname = NULL, *mname = NULL;
-  int ypbind_version = 2;
+  int ypbind_version = -1;
 
   setlocale (LC_MESSAGES, "");
   setlocale (LC_CTYPE, "");
@@ -177,6 +293,7 @@ main (int argc, char **argv)
       {
         {"version", no_argument, NULL, '\255'},
         {"usage", no_argument, NULL, '\254'},
+	{"verbose", no_argument, NULL, '\253'},
         {"help", no_argument, NULL, '?'},
         {NULL, 0, NULL, '\0'}
       };
@@ -199,7 +316,7 @@ main (int argc, char **argv)
 	case 'V':
 	  Vflag = 1;
 	  ypbind_version = atoi (optarg);
-	  if (ypbind_version < 1 || ypbind_version > 2)
+	  if (ypbind_version < 1 || ypbind_version > 3)
 	    {
 	      print_error ();
 	      return 1;
@@ -208,6 +325,9 @@ main (int argc, char **argv)
 	case 'x':
 	  xflag = 1;
 	  break;
+	case '\253':
+	  verbose = 1;
+	  break;
 	case '?':
 	  print_help ();
 	  return 0;
@@ -264,6 +384,12 @@ main (int argc, char **argv)
 		       "ypwhich", yperr_string (error));
 	      return 1;
 	    }
+	  if (domainname == NULL || strlen (domainname) == 0)
+	    {
+	      fprintf (stderr, _("%s: no local yp domain set\n"),
+		       "ypwhich");
+	      return 1;
+	    }
 	}
 
       if (mflag)
@@ -338,35 +464,11 @@ main (int argc, char **argv)
 	}
       else
 	{
-	  struct sockaddr_in sock_in;
-	  struct hostent *host;
+	  if (!hflag)
+	    hostname = "localhost";
 
-	  memset (&sock_in, 0, sizeof sock_in);
-	  sock_in.sin_family = AF_INET;
-	  if (hflag)
-	    {
-	      if (inet_aton (hostname, &sock_in.sin_addr) == 0)
-		{
-		  host = gethostbyname (hostname);
-		  if (!host)
-		    {
-		      fprintf (stderr, _("ypwhich: host %s unknown\n"),
-			       hostname);
-		      return 1;
-		    }
-		  memcpy (&sock_in.sin_addr, host->h_addr_list[0],
-			  sizeof (sock_in.sin_addr));
-		}
-	      if (print_bindhost (domainname, &sock_in, ypbind_version))
-		return 1;
-	    }
-	  else
-	    {
-	      sock_in.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-
-	      if (print_bindhost (domainname, &sock_in, ypbind_version))
-		return 1;
-	    }
+	  if (print_bindhost (hostname, domainname, ypbind_version))
+	    return 1;
 	}
     }
 
